//
// frmMain.cpp
// TfrmMain implementation
//

#include "frmColorPicker.h"

// [FORM_IMPL_INCLUDES]
#include "frmMain.h"
#include "frmMain.fcp"
// [END_FORM_IMPL_INCLUDES]

#include <Application.h>
#include <Roster.h>
#include <Path.h>

#include <Screen.h>
#include <MsgBox.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <fstream>

enum {
  cbtnActiveTab_click = 0x01
, cbtnFrame_click
, cbtnActiveBorder_click
, cbtnInactiveBorder_click
, cbtnActiveTitle_click
, cbtnInactiveTitle_click
};

struct clrmap {
  int32 what;
  TfrmColorPicker *form;
  ColorButton *button;
  BString title;
  BString param;
};

const int MAPSIZE = 6;
clrmap _map[MAPSIZE] = {
   { cbtnActiveTab_click,       0, 0, "Active Tab Color",     "activetab"     }
,  { cbtnFrame_click,           0, 0, "Frame Color",       "frame"     }
,  { cbtnActiveBorder_click,    0, 0, "Active Border Color",   "activeborder"     }
,  { cbtnInactiveBorder_click,  0, 0, "Inactive Border Color",   "inactiveborder"     }
,  { cbtnActiveTitle_click,     0, 0, "Active Title Color",   "activetitle"     }
,  { cbtnInactiveTitle_click,   0, 0, "Inactive Title Color",   "inactivetitle"     }
};

void CenterWindowOnScreen(BWindow* w)
{
  BRect  screenFrame = (BScreen(B_MAIN_SCREEN_ID).Frame());
  BPoint   pt;
  pt.x = screenFrame.Width()/2 - w->Bounds().Width()/2;
  pt.y = screenFrame.Height()/2 - w->Bounds().Height()/2;

  if (screenFrame.Contains(pt))
    w->MoveTo(pt);
}


void TfrmMain::MessageReceived(BMessage *msg)
{
  switch(msg->what)
  {
  case cbtnActiveTab_click:
  case cbtnFrame_click:
  case cbtnActiveBorder_click:
  case cbtnInactiveBorder_click:
  case cbtnActiveTitle_click:
  case cbtnInactiveTitle_click:
    ShowClrEditor(msg->what);
    break;

    default:
      TForm::MessageReceived(msg);
      break;
  }
}

void TfrmMain::ShowClrEditor(int32 what)
{
  TfrmColorPicker *frm = PickerFromIndex(what);
  if(frm)
  {
    // set the form's button to the appropriate one,
    // and show the form.
    frm->SetClrButton(ButtonFromIndex(what));
    frm->SetTitle(TitleFromIndex(what).String());
    frm->Show();
  }
}

TfrmColorPicker *TfrmMain::PickerFromIndex(int32 what)
{
  TfrmColorPicker *frm = 0;
  for(int32 i = 0; i < MAPSIZE; i++)
  {
    if(_map[i].what == what)
    {
      frm = _map[i].form;
      break;
    }
  }
  if(!frm)
  {
    frm = new TfrmColorPicker();
  }
  return frm;
}

ColorButton *TfrmMain::ButtonFromIndex(int32 what)
{
  ColorButton *btn = 0;
  for(int32 i = 0; i < MAPSIZE; i++)
  {
    if(_map[i].what == what)
    {
      btn = _map[i].button;
      break;
    }
  }
  return btn;
}

BString TfrmMain::TitleFromIndex(int32 what)
{
  BString s = "";
  for(int32 i = 0; i < MAPSIZE; i++)
  {
    if(_map[i].what == what)
    {
      s = _map[i].title;
      break;
    }
  }
  return s;
}

void TfrmMain::SetRectBy(BRect &r, TStringView<TfrmMain> *sv)
{
  r = sv->Frame();
  r.left = r.right + 40;
  r.right = r.left + 35;
}

#define setcolor(r,g,b) { clr.red = r; clr.green = g; clr.blue = b; }

void TfrmMain::FormCreate(void)
{
  BRect r;
  rgb_color clr;
  
  SetRectBy(r, svActiveTab);
  cbtnActiveTab = new ColorButton(r, "cbtnActiveTab", "", new BMessage(cbtnActiveTab_click));
  setcolor(0xff,0xcb,0x0);
  cbtnActiveTab->SetBGColor(clr);
  
  SetRectBy(r, svFrame);
  cbtnFrame = new ColorButton(r, "cbtnFrame", "", new BMessage(cbtnFrame_click));
  setcolor(0x98,0x98,0x98);
  cbtnFrame->SetBGColor(clr);

  SetRectBy(r, svActiveBorder);
  cbtnActiveBorder = new ColorButton(r, "cbtnActiveBorder", "", new BMessage(cbtnActiveBorder_click));
  setcolor(0xd8,0xd8,0xd8);
  cbtnActiveBorder->SetBGColor(clr);
  
  SetRectBy(r, svInactiveBorder);
  cbtnInactiveBorder = new ColorButton(r, "cbtnInactiveBorder", "", new BMessage(cbtnInactiveBorder_click));
  setcolor(0xeb,0xeb,0xeb);
  cbtnInactiveBorder->SetBGColor(clr);
  
  SetRectBy(r, svActiveTitle);
  cbtnActiveTitle = new ColorButton(r, "cbtnActiveTitle", "", new BMessage(cbtnActiveTitle_click));
  setcolor(0x0,0x0,0x0);
  cbtnActiveTitle->SetBGColor(clr);

  SetRectBy(r, svInactiveTitle);
  cbtnInactiveTitle = new ColorButton(r, "cbtnInactiveTitle", "", new BMessage(cbtnInactiveTitle_click));
  setcolor(0x80,0x80,0x80);
  cbtnInactiveTitle->SetBGColor(clr);

  Box1->AddChild(cbtnActiveTab);
  Box1->AddChild(cbtnFrame);
  Box1->AddChild(cbtnActiveBorder);
  Box1->AddChild(cbtnInactiveBorder);
  Box1->AddChild(cbtnActiveTitle);
  Box1->AddChild(cbtnInactiveTitle);
  
  int32 i = 0;
  _map[i++].button = cbtnActiveTab;
  _map[i++].button = cbtnFrame;
  _map[i++].button = cbtnActiveBorder;
  _map[i++].button = cbtnInactiveBorder;
  _map[i++].button = cbtnActiveTitle;
  _map[i++].button = cbtnInactiveTitle;
  
  CenterWindowOnScreen(this);
}

void TfrmMain::FormDestroy(void)
{

}

void RGBToHex(rgb_color &clr, char *buf)
{
  char s[80] = "";
  sprintf(s, "%02x", clr.red);
  strcat(buf, s);
  sprintf(s, "%02x", clr.green);
  strcat(buf, s);
  sprintf(s, "%02x", clr.blue);
  strcat(buf, s);
}

BString AppPath(void)
{
	BPath path;
	app_info info;
	be_app->GetAppInfo(&info); 
	BEntry entry(&info.ref); 
	entry.GetPath(&path); 
	path.GetParent(&path);
	BString s(path.Path());
	return s;
}

void TfrmMain::btnApplyInvoked(void)
{
  char buf[80] = "";
  rgb_color clr;
  
  // Build a command line to pass to WindowShade
  BString s = "#!/bin/sh\n\n";
  
  s << "# Generated by WSFrontEnd (C) 2000, Kelly Schrock\n";
  
  s << AppPath();
  s << "/WindowShade";
  
  for(int32 i = 0; i < MAPSIZE; i++)
  {  
    clr = _map[i].button->BGColor();
    buf[0] = 0;
    RGBToHex(clr, buf);
    if(i > 0) s << " \\\n";
    s << " -" << _map[i].param << " " << buf;
  }
  
  s << " \\\n -refresh\n";
  
  BString shellscript = AppPath();
  shellscript << "/setcolors";
  
  ofstream out(shellscript.String());
  out << s.String();
  out.close();
  
  // try to execute it!
  system(shellscript.String());
}
